import pytest
import asyncio
from agenix.rate_limiter import RedisRateLimiter, RedisRateLimiterArgs
import time


@pytest.fixture
def limiter():
    """Создает экземпляр лимитера для тестов"""
    limiter = RedisRateLimiter(
        host="localhost",
        port=6379,
        db=0,
        limit=10,
        window=1
    )
    limiter.redis.flushdb()
    yield limiter
    limiter.redis.flushdb()


def test_basic_rate_limiting(limiter):
    """Тест базового ограничения скорости"""
    client_id = "test_client"

    # Проверяем начальное количество токенов
    assert limiter.get_remaining_tokens(client_id) == 10

    # Пробуем взять все токены
    for _ in range(10):
        assert limiter.acquire(client_id)

    # Проверяем что токены закончились
    assert not limiter.acquire(client_id)
    assert limiter.get_remaining_tokens(client_id) == 0


def test_token_refill(limiter):
    """Тест пополнения токенов"""
    client_id = "test_client"

    # Берем все токены
    for _ in range(10):
        assert limiter.acquire(client_id)

    # Ждем пополнения
    time.sleep(0.5)

    # Проверяем что токены начали пополняться
    tokens = limiter.get_remaining_tokens(client_id)
    assert 0 < tokens <= 5


def test_temporary_limits(limiter):
    """Тест временных ограничений"""
    client_id = "test_client"

    # Добавляем временное ограничение
    limiter.add_temporary_limit(client_id, limit=5, duration=1)

    # Проверяем что лимит применился
    assert limiter.get_current_limit(client_id) == 5
    assert limiter.get_remaining_tokens(client_id) == 5

    # Берем все доступные токены
    for _ in range(5):
        assert limiter.acquire(client_id)

    # Проверяем что больше взять нельзя
    assert not limiter.acquire(client_id)

    # Ждем истечения временного правила
    time.sleep(1.1)

    # Проверяем что вернулся базовый лимит
    assert limiter.get_current_limit(client_id) == 10
    assert 0 < limiter.get_remaining_tokens(client_id) <= 10


def test_multiple_temporary_limits(limiter):
    """Тест множественных временных ограничений"""
    client_id = "test_client"

    # Добавляем несколько временных ограничений
    limiter.add_temporary_limit(client_id, limit=8, duration=2)
    limiter.add_temporary_limit(client_id, limit=5, duration=1)

    # Проверяем что применился самый строгий лимит
    assert limiter.get_current_limit(client_id) == 5
    assert limiter.get_remaining_tokens(client_id) == 5

    # Ждем истечения первого правила
    time.sleep(1.1)

    # Проверяем что применилось следующее правило
    assert limiter.get_current_limit(client_id) == 8
    assert limiter.get_remaining_tokens(client_id) <= 8


def test_clear_limits(limiter):
    """Тест очистки правил"""
    client_id = "test_client"

    # Добавляем временное ограничение
    limiter.add_temporary_limit(client_id, limit=5, duration=10)
    assert limiter.get_current_limit(client_id) == 5

    # Очищаем правила
    limiter.clear_limits(client_id)

    # Проверяем что вернулся базовый лимит
    assert limiter.get_current_limit(client_id) == 10
    assert limiter.get_remaining_tokens(client_id) == 10
